@@ -1,13 +1,13 @@
 ;;; url.el --- Uniform Resource Locator retrieval tool
 ;; Author: wmperry
-;; Created: 1998/12/30 11:26:49
-;; Version: 1.9
+;; Created: 1999/11/12 16:30:06
+;; Version: 1.13
 ;; Keywords: comm, data, processes, hypermedia
 
 ;;; LCD Archive Entry:
 ;;; url|William M. Perry|wmperry@cs.indiana.edu|
 ;;; Functions for retrieving/manipulating URLs|
-;;; 1998/12/30 11:26:49|1.9|Location Undetermined
+;;; 1999/11/12 16:30:06|1.13|Location Undetermined
 ;;;
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -103,9 +103,7 @@
 (autoload 'url-news "url-news")
 (autoload 'url-nntp "url-news")
 (autoload 'url-cid "url-cid")
-
-(if (featurep 'ldap)
-    (autoload 'url-ldap "url-ldap"))
+(autoload 'url-ldap "url-ldap")
 
 (autoload 'url-open-stream "url-gw")
 (autoload 'url-mime-response-p "url-http")
@@ -332,6 +330,11 @@
     (cond
      ((and url-current-content-length (> url-current-content-length 1)
 	   url-current-mime-type)
+      (if (fboundp 'progress)
+	  (progress "Reading [%s] %s"
+		    (url-percentage current-length
+				    url-current-content-length)
+		    url-current-mime-type (url-pretty-length url-current-content-length)))
       (url-lazy-message "Reading [%s]... %s of %s (%d%%)"
 			url-current-mime-type
 			(url-pretty-length current-length)
@@ -339,6 +342,11 @@
 			(url-percentage current-length
 					url-current-content-length)))
      ((and url-current-content-length (> url-current-content-length 1))
+      (if (fboundp 'progress)
+	  (progress "Reading %s"
+		    (url-percentage current-length
+				    url-current-content-length)
+		    url-current-mime-type (url-pretty-length url-current-content-length)))
       (url-lazy-message "Reading... %s of %s (%d%%)"
 			(url-pretty-length current-length)
 			(url-pretty-length url-current-content-length)
@@ -481,9 +489,11 @@ The return value is a cons of the url and the date last accessed as a string"
   "Returns t iff the specified URL is directly accessible
 on your filesystem.  (nfs, local file, etc)."
   (let* ((urlobj (if (vectorp url) url (url-generic-parse-url url)))
-	 (type (url-type urlobj)))
+	 (type (url-type urlobj))
+	 (host (url-host urlobj)))
     (and (member type '("file" "ftp"))
-	 (not (url-host urlobj)))))
+	 (url-host-is-local-p host)
+	 (not (string-match "\\.\\(gz\\|Z\\|z\\)$" url)))))
 
 ;;;###autoload
 (defun url-file-attributes (url &rest args)
@@ -748,24 +758,30 @@ numbers, etc."
 		name (format "%s<%d>" start x)))
 	name))))
 
+(defvar url-generate-unique-filename-counter 0)
+
 (defun url-generate-unique-filename (&optional fmt)
   "Generate a unique filename in url-temporary-directory"
+  (setq url-generate-unique-filename-counter
+	(1+ url-generate-unique-filename-counter))
   (if (not fmt)
       (let ((base (format "url-tmp.%d" (user-real-uid)))
 	    (fname "")
-	    (x 0))
+	    (x url-generate-unique-filename-counter))
 	(setq fname (format "%s%d" base x))
 	(while (file-exists-p (expand-file-name fname url-temporary-directory))
 	  (setq x (1+ x)
 		fname (concat base (int-to-string x))))
+	(setq url-generate-unique-filename-counter x)
 	(expand-file-name fname url-temporary-directory))
     (let ((base (concat "url" (int-to-string (user-real-uid))))
 	  (fname "")
-	  (x 0))
+	  (x url-generate-unique-filename-counter))
       (setq fname (format fmt (concat base (int-to-string x))))
       (while (file-exists-p (expand-file-name fname url-temporary-directory))
 	(setq x (1+ x)
 	      fname (format fmt (concat base (int-to-string x)))))
+      (setq url-generate-unique-filename-counter x)
       (expand-file-name fname url-temporary-directory))))
 
 (defun url-lazy-message (&rest args)
@@ -929,8 +945,7 @@ dumped with emacs."
     (url-register-protocol 'telnet)
     (url-register-protocol 'tn3270)
     (url-register-protocol 'proxy)
-    (if (featurep 'ldap)
-	(url-register-protocol 'ldap))
+    (url-register-protocol 'ldap)
     (url-register-protocol 'auto 'url-handle-no-scheme)
 
     ;; Register all the authentication schemes we can handle
@@ -989,7 +1004,7 @@ dumped with emacs."
 				     noproxy "") "\\)"))
 		      url-proxy-services))))
 
-    ;; Set the password entry funtion based on user defaults or guess
+    ;; Set the password entry function based on user defaults or guess
     ;; based on which remote-file-access package they are using.
     (cond
      (url-passwd-entry-func nil)	; Already been set
@@ -1040,10 +1055,13 @@ dumped with emacs."
     (run-hooks 'url-load-hook)
     (setq url-setup-done t)))
 
+;;; Evidently, there is a chicken and egg problem with the
+;;; eval-when-compile in url-get-url-at-point, so we have to eval this
+;;; at compile time.
 (eval-and-compile
-(defvar url-get-url-filename-chars "-%.?@a-zA-Z0-9()_/:~=&"
-  "Valid characters in a URL")
-)
+  (defvar url-get-url-filename-chars "-%.?@a-zA-Z0-9()_/:~=&"
+    "Valid characters in a URL")
+  )
 
 ;;;###autoload
 (defun url-get-url-at-point (&optional pt)
@@ -1747,7 +1765,12 @@ user for what type to save as."
 	 ((looking-at "(setq") (url-write-Emacs-history fname))
 	 (t (url-write-Emacs-history fname)))
 	(kill-buffer (current-buffer))))))
-  (setq url-history-changed-since-last-save nil))
+  (setq url-history-changed-since-last-save nil)
+  (setq url-history-changed-since-last-save nil)
+  (if (not (or (eq url-privacy-level 'paranoid)
+ 	       (and (listp url-privacy-level) (memq 'cookies url-privacy-level))))
+      (url-cookie-write-file))
+  (w3-write-explicit-encodings))
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -2001,6 +2024,7 @@ retrieve a URL by its HTML source."
 	  url (if cached (url-cache-create-filename url) url))
     (save-excursion
       (set-buffer (get-buffer-create url-working-buffer))
+      (make-local-variable 'track-mouse)
       (if (boundp 'buffer-file-coding-system)
 	  (setq buffer-file-coding-system nil))
       (setq url-current-can-be-cached (not no-cache)
