@@ -1,7 +1,7 @@
 ;;; w3-display.el --- display engine
 ;; Author: wmperry
-;; Created: 1998/12/28 21:44:07
-;; Version: 1.8
+;; Created: 1999/11/14 01:37:16
+;; Version: 1.17
 ;; Keywords: faces, help, hypermedia
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -32,6 +32,7 @@
 (require 'w3-keyword)
 (require 'css)
 (require 'font)
+(require 'url-parse)
 (require 'w3-widget)
 (require 'w3-imap)
 
@@ -139,6 +140,17 @@
 
 (make-variable-buffer-local 'w3-last-fill-pos)
 
+(defadvice widget-convert-text (around emacspeak pre act comp)
+  "Protect value of personality if set originally"
+  (let ((start (ad-get-arg 1))
+        (end (ad-get-arg 2))
+        (orig nil ))
+    (setq orig (get-text-property start 'personality))
+    ad-do-it 
+    (and orig 
+         (put-text-property start end 
+                            'personality orig))))
+
 (defconst w3-fill-prefixes-vector
   (let ((len 0)
         (prefix-vector (make-vector 80 nil)))
@@ -265,11 +277,13 @@
       (or (if backgroundp
 	      (face-background-name 'default)
 	    (face-foreground-name 'default))
-	  (frame-property nil (if backgroundp 'background-color 'foreground-color)))
+	  (frame-property nil (if backgroundp 'background-color 'foreground-color))
+	  (if backgroundp "white" "black"))
       (or (if backgroundp
 	      (face-background 'default)
 	    (face-foreground 'default))
-	  (frame-parameter nil (if backgroundp 'background-color 'foreground-color)))))
+	  (frame-parameter nil (if backgroundp 'background-color 'foreground-color))
+	  (if backgroundp "white" "black"))))
 
 (defun w3-display-background-useless-p (color)
   (let ((foreground-color (font-color-rgb-components
@@ -397,7 +411,7 @@ If the face already exists, it is unmodified."
        (= w3-last-fill-pos (point))
        (> w3-last-fill-pos (point-max)))
       (if (not (eq (char-before (point)) ?\n))
-	  (setq n (1+ n))) ; at least put one line in
+ 	  (setq n (1+ n))) ; at least put one line in
     (let ((fill-column (max (1+ (length fill-prefix)) fill-column))
 	  width)
       (case (car w3-display-alignment-stack)
@@ -476,7 +490,8 @@ If the face already exists, it is unmodified."
 			   (list 'personality (car w3-active-voices))))
   )
 
-(require 'mule-sysdp)
+(eval-and-compile
+  (require 'mule-sysdp))
 
 (defun w3-display-get-cookie (args)
   (if (not (fboundp 'cookie))
@@ -509,7 +524,7 @@ If the face already exists, it is unmodified."
     (set-buffer (or (w3-widget-buffer widget) (current-buffer)))
     (let* ((url (widget-get widget :href))
 	   (name (widget-get widget :name))
-	   (text (buffer-substring-no-properties (widget-get widget :from)
+	   (text (buffer-substring (widget-get widget :from)
 						 (widget-get widget :to)))
 	   (title (widget-get widget :title))
 	   (check w3-echo-link)
@@ -773,11 +788,12 @@ If the face already exists, it is unmodified."
 		      (error 'nothing)))))
       (setq w3-image-widgets-waiting (cons widget w3-image-widgets-waiting)))
      ((or w3-delay-image-loads		; Delaying images
-	  (not (fboundp 'valid-specifier-domain-p)) ; Can't do images
+	  (and (not (fboundp 'valid-specifier-domain-p)) ; Can't do images (XEmacs)
+	       (not (boundp 'image-types))) ; Can't do images (Emacs)
 	  (eq (device-type) 'tty))	; Why bother?
       (w3-add-delayed-graphic widget))
      ((not (w3-image-loadable-p src nil)) ; Hey, we can't load it!
-      (mesage "Skipping image %s" (url-basepath src t))
+      (message "Skipping image %s" (url-basepath src t))
       (w3-add-delayed-graphic widget))
      (t					; Grab the images
       (let (
@@ -944,7 +960,7 @@ If the face already exists, it is unmodified."
 (defmacro w3-image-alt (src)
   (`
    (let* ((doc-alt (w3-get-attribute 'alt))
-	  (alt (or doc-alt
+	  (alt (or (and (stringp doc-alt) (string-match "[^ \t\n]" doc-alt) doc-alt)
 		   (cond
 		    ((null w3-auto-image-alt) "")
 		    ((eq t w3-auto-image-alt)
@@ -985,6 +1001,8 @@ If the face already exists, it is unmodified."
 			 :href href	   ; Hyperlink destination
 			 :target target	   ; target frame
 			 :button-face face ; img:link or img:visited entry in stylesheet
+			 'row w3-display-current-row
+			 'column w3-display-current-col
 			 )))
        (setq widget (apply (function widget-create) (cadr hyperimage-info)))
        (widget-put widget 'buffer (current-buffer))
@@ -1928,7 +1946,8 @@ Format: (((image-alt row column) . offset) ...)")
 	    (setq nd (or (next-single-property-change st 'w3-hyperimage-info)
 			 (point-max)))
 	    (let* ((raw-alt (widget-get info 'alt))
-		   (drawn-alt (if (and (stringp raw-alt) (string-match "\\([^ \t\n]+\\)[ \t\n]" raw-alt))
+		   ;; remove trailing blanks:
+		   (drawn-alt (if (and (stringp raw-alt) (string-match "\\(.*[^ \t\n]+\\)[ \t\n]+$" raw-alt))
 				  (substring raw-alt 0 (match-end 1))
 				raw-alt))
 		   (row (widget-get info 'row))
@@ -1945,14 +1964,16 @@ Format: (((image-alt row column) . offset) ...)")
 		  (delete-region st (+ st offset)))
 	      (setq to-cut (- (point-max) max)
 		    new-offset (1+ (- nd st)))
-	      (cond ((not (zerop to-cut))
+	      (cond ((> to-cut 0)
 		     ;; cut end of drawn-alt if too long after resurrection
 		     (delete-region nd (+ nd to-cut))
 		     (if offset-elt
 			 (setcdr offset-elt (+ offset new-offset))
 		       (setq w3-resurrect-images-offset
 			     (cons (cons (list drawn-alt row col) new-offset)
-				   w3-resurrect-images-offset))))))
+				   w3-resurrect-images-offset))))
+		    ((< to-cut 0)
+		     (insert (make-string (- to-cut) ? )))))
 	    (widget-put widget 'buffer (current-buffer))
 	    (w3-maybe-start-image-download widget)
 	    (if (widget-get widget :from)
@@ -1962,7 +1983,15 @@ Format: (((image-alt row column) . offset) ...)")
       (setq st (next-single-property-change st 'w3-hyperimage-info)))
     (setq w3-resurrect-images-offset nil)))
 
+(require 'w3-mouse)
+(defvar w3-display-hackmap nil "Keymap used for hyperlink widgets")
+  
 (defun w3-resurrect-hyperlinks ()
+  (if (and (not w3-display-hackmap) w3-running-xemacs)
+      (progn
+	(setq w3-display-hackmap (make-sparse-keymap))
+	(set-keymap-parent w3-display-hackmap widget-button-keymap)
+	(define-key w3-display-hackmap (vector w3-mouse-button3) 'w3-popup-menu)))
   (let ((st (point-min))
 	(inhibit-read-only t)
 	info nd node face)
@@ -1973,6 +2002,7 @@ Format: (((image-alt row column) . offset) ...)")
 			 (point-max)))
 	    (apply 'widget-convert-text 'link st nd st nd (nconc
 							   (list :start st
+								 :button-keymap w3-display-hackmap
 								 :end nd)
 							   info))))
       (setq st (next-single-property-change st 'w3-hyperlink-info)))))
@@ -1986,6 +2016,22 @@ Format: (((image-alt row column) . offset) ...)")
 	    rval (plist-put rval newsym (cdr cur))))
     rval))
 
+(defvar w3-auto-run-java nil
+  "*If non-nil, will cause Java applets to automatically be run in another process.")
+
+(defun w3-display-handle-java (node)
+  (let ((options (nth 1 node))
+	(params (mapcar (lambda (subnode)
+			  (if (eq (car-safe subnode) 'param)
+			      (cons (cdr-safe (assq 'name (nth 1 subnode)))
+				    (cdr-safe (assq 'value (nth 1 subnode))))))
+			(nth 2 node))))
+    (setq options (delq nil options)
+	  params (delq nil params))
+    (if (not (assq 'codebase options))
+	(push (cons 'codebase (url-view-url t)) options))
+    (w3-java-run-applet options params)))
+
 (defun w3-display-node (node &optional nofaces)
   (let (
 	(content-stack (list (list node)))
@@ -2197,7 +2243,8 @@ Format: (((image-alt row column) . offset) ...)")
 	     (if w3-display-frames
 		 (let ((frames (nth 2 node))
 		       (frameset-cardinal 0)
-		       (cols (cdr-safe (assq 'cols args))))
+		       (cols (cdr-safe (assq 'cols args)))
+		       (rows (cdr-safe (assq 'rows args))))
 		   (while (and frames (memq (car (car frames)) '(frame frameset)))
 		      (setq frameset-cardinal (1+ frameset-cardinal)
 			    frames (cdr frames)))
@@ -2205,7 +2252,8 @@ Format: (((image-alt row column) . offset) ...)")
 			       frameset-cardinal
 			       (if (w3-frameset-dimensions-p cols)
 				   (assq 'cols args)
-				 (assq 'rows args)))
+				 (if (w3-frameset-dimensions-p rows)
+				     (assq 'rows args))))
 			 w3-frameset-structure)
 		   (w3-handle-content node))
 	       (w3-handle-content node)))
@@ -2233,8 +2281,9 @@ Format: (((image-alt row column) . offset) ...)")
 		 (w3-handle-empty-tag)
 	       (w3-handle-content node)))
 	    (applet			; Wow, Java
-	     (w3-handle-content node)
-	     )
+	     (if w3-auto-run-java
+		 (w3-display-handle-java node)
+	       (w3-handle-content node)))
 	    (script			; Scripts
 	     (w3-handle-empty-tag))
 	    ((embed object)		; Embedded images/content
@@ -2646,7 +2695,7 @@ Format: (((image-alt row column) . offset) ...)")
   (let ((inhibit-read-only t))
     (save-excursion
       (goto-char (point-min))
-      (while (re-search-forward "[ \t]*\n[ \t]*" nil t)
+      (while (re-search-forward "\n" nil t)
 	(remove-text-properties (match-beginning 0) (match-end 0)
 				'(face nil mouse-face nil) nil)))))
 
@@ -2769,7 +2818,7 @@ Format: (((image-alt row column) . offset) ...)")
       (setq-default url-be-asynchronous old-asynch))))
 
 (defun w3-frameset-dimensions-p (str)
-  (and str (not (string-equal str "*")) (not (string-equal str "100%"))))
+  (and str (not (string-equal str "*")) (not (string-match "100%" str))))
 
 (defun w3-display-frameset (frameset-structure)
   (let* ((structure frameset-structure)
@@ -2782,7 +2831,8 @@ Format: (((image-alt row column) . offset) ...)")
 			w3-tty-char-width))
 	 (char-height (if (> (frame-char-height) 1)
 			  (frame-char-height)
-			w3-tty-char-height)))
+			w3-tty-char-height))
+	 (inhibit-frame nil))
     (pop structure)
     ;; columns ?
     (if (w3-frameset-dimensions-p cols)
@@ -2793,32 +2843,58 @@ Format: (((image-alt row column) . offset) ...)")
 	  (setq rows (w3-decode-frameset-dimensions
 		      rows (window-height) window-min-height char-height))
 	;; default: columns of equal width
-	(let ((fwidth (/ (window-width) frameset-cardinal)))
-	  (while (> frameset-cardinal 0)
+	(let ((fwidth (/ (window-width) frameset-cardinal))
+	      (cardinal frameset-cardinal))
+	  (while (> cardinal 0)
 	    (push fwidth cols)
-	    (setq frameset-cardinal (1- frameset-cardinal))))))
+	    (setq cardinal (1- cardinal))))))
     (while (> frameset-cardinal 0)
       (cond ((cdr cols)
-	     (split-window-horizontally (car cols))
+	     (if (or (< (car cols) window-min-width)
+		     (< (- (window-width) (car cols)) window-min-width))
+		 (setq inhibit-frame (format "Width %d" (window-width)))
+	       (split-window-horizontally (car cols)))
 	     (pop cols))
 	    ((cdr rows)
-	     (split-window-vertically (car rows))
+	     (if (or (< (car rows) window-min-height)
+		     (< (- (window-height) (car rows)) window-min-height))
+		 (setq inhibit-frame (format "Height %d" (window-height)))
+	       (split-window-vertically (car rows)))
 	     (pop rows)))
       (cond ((eq (car (car structure)) 'frame)
 	     (let ((href (nth 2 (car structure)))
 		   (name (nth 1 (car structure)))
 		   (url-working-buffer url-default-working-buffer) ; in case url-multiple-p is t
-		   (w3-notify 'semibully))
+		   (w3-notify 'semibully)
+		   (next-frame-window (next-window)))
 	       (pop structure)
-	       (w3-fetch href)
-	       (let ((buf (current-buffer)))
-		 (set-buffer (url-buffer-visiting href))
-		 (setq w3-frame-name name
-		       w3-target-window-distances nil)
-		 (set-buffer buf))
-	       (other-window 1)))
+	       (cond (inhibit-frame
+		      (w3-warn 'html (format "%s insufficient to split windows for HTML frame \"%s\""
+					     inhibit-frame name))
+		      (setq inhibit-frame nil
+			    next-frame-window (selected-window)))
+		     (t
+		      (w3-fetch href)))
+	       (let ((buf (current-buffer))
+		     (framebuf (url-buffer-visiting href)))
+		 (cond (framebuf
+			(set-buffer framebuf)
+			(setq w3-frame-name name
+			      w3-target-window-distances nil)
+			(set-buffer buf)
+			(select-window next-frame-window))))))
 	    ((eq (car (car structure)) 'frameset)
-	     (setq structure (w3-display-frameset structure))))
+	     (cond (inhibit-frame
+		    (w3-warn 'html (format "%s insufficient to split windows for HTML frameset"
+					   inhibit-frame))
+		    (let ((sub-frameset-cardinal (cadr (car structure))))
+		      (pop structure)
+		      (while (> sub-frameset-cardinal 0)
+			(pop structure)
+			(setq sub-frameset-cardinal (1- sub-frameset-cardinal))))
+		    (setq inhibit-frame nil))
+		   (t
+		    (setq structure (w3-display-frameset structure))))))
       (setq frameset-cardinal (1- frameset-cardinal)))
     structure))
 
@@ -2869,12 +2945,19 @@ Format: (((image-alt row column) . offset) ...)")
 				  dimensions))
 			   (t
 			    ;; absolute number: pixel height
-			    (push (max (1+ (/ (car (read-from-string match))
-					      pixel-dim))
-				       min-dim)
-				  dimensions)))
-		     (setq remaining-available-dimension
-			   (- remaining-available-dimension (car dimensions)))))))
+			    (let* ((dim-in-pixels (car (read-from-string match)))
+				   (dim (max (/ dim-in-pixels pixel-dim)
+					     min-dim)))
+			      (cond ((<= dim remaining-available-dimension)
+				     (push dim dimensions)
+				     (setq remaining-available-dimension
+					   (- remaining-available-dimension (car dimensions))))
+				    (t
+				     (w3-warn 'html (format "Frame dimension too large: %d" dim-in-pixels))
+				     ;; too large: replace with *
+				     (push '* dimensions)
+				     (setq nb-stars (1+ nb-stars)
+					   norm-stars (1+ norm-stars)))))))))))
 	  (if (zerop nb-stars)
 	      ;; push => reverse order
 	      (reverse dimensions)
